
#include "pdatabasefile.h"

#include "../cavist/precord.h"
#include "../cavist/pbottle.h"


#include "stdafx.h"


void logLine(const char* str);
void logLine(const CString& str);

class PBottle;




//------------------------------------------------------------------------
PFileRecord::PFileRecord()
    :data_(0),
	attributes_(0),
	localChunkID_(0)
{
	uniqueID_[0] = 0;
	uniqueID_[1] = 0;
	uniqueID_[2] = 0;
}

PFileRecord::~PFileRecord()
{
    delete [] data_;
    data_ = 0;
}

void PFileRecord::setData(char* data, long len)
{
	data_ = data;
	*(long*)&localChunkID_ = len;
}






//------------------------------------------------------------------------
void removePFileRecord(void* elem)
{
	delete (PFileRecord*)elem;
}

PDatabaseFile::PDatabaseFile()
{
}

PDatabaseFile::~PDatabaseFile()
{
}

long PDatabaseFile::readLong(CFile& file)
{
	BYTE val[4];
    DWORD aval;
	file.Read(val, 4);

    aval = (DWORD)(val[0] << 24) + (DWORD)(val[1] << 16) + (DWORD)(val[2] << 8) + (DWORD)val[3];

	return aval;
}

BYTE PDatabaseFile::readByte(CFile& file)
{
	BYTE val;
	file.Read(&val, 1);
	return val;
}
short PDatabaseFile::readShort(CFile& file)
{
	short aval;
  unsigned char val[2];
	file.Read(val, 2);

  unsigned char a = val[0];
  unsigned char b = val[1];

  signed long c = (signed long)(a << 8);
  signed long d = (signed long)b;

  aval = (unsigned short)(c + d);

  return aval;
}

Boolean PDatabaseFile::readString(CFile& file, char* buf, long nb)
{
	if (nb != (long)file.Read(buf, nb))
		return false;

	return true;
}

void doNothing(void* c)
{
}

Boolean PDatabaseFile::read(PCollection& coll, const PString& name)
{
	CFile file;

	long numRecords;

	if (!file.Open(name.data(), CFile::modeRead))
	{
		return false;
	}

	
	// DatabaseHdrType
	if (!readString(file, name_, 32))
	{
		return false;
	}
	attributes_ =		readShort(file);
	version_ =			readShort(file);
	creationDate_ =		readLong(file);
	modificationDate_ =	readLong(file);
	lastBackupDate_ =	readLong(file);
	modificationNumber_ =	readLong(file);
	appInfoID_ =		readLong(file);
	sortInfoID_ =		readLong(file);
	
	CString str, str2;
	str = "BOTL";
	if (!readString(file, type_, 4))
	{
		return false;
	}
  type_[4] = 0;
	str2 = type_;
	//str2.SetAt(4, 0);
	if ( str != str2 )
	{
		logLine("The Type is not 'BOTL'");
		return false;
	}

	if (!readString(file, creator_, 4))
	{
		return false;
	}
  creator_[4] = 0;
	str = "CVST";
	str2 = creator_;
	//str2.SetAt(4, 0);
	if ( str != str2 )
	{
		logLine("The creator ID is not 'CVST'");
		return false;
	}


	uniqueIDSeed_ =		readLong(file); //72

	// RecordListType
	nextRecordListID_ = readLong(file);
	numRecords = readShort(file);
	if (appInfoID_ != 0)
	{
		ASSERT(0);
		//PFileRecord* rec = new PFileRecord;
		//rec->localChunkID_ = appInfoID_;
		//coll.add(rec);
	}

  if (numRecords < 0)
  {
    CString str;
    str.Format("The number of records (%d) is not right.", numRecords);
    logLine(str);
    return false;
  }

	PCollection recSizes(doNothing, 0);
	char uniqueID[4];
	long offset = 0;
	long lastOffset = readLong(file);
	readByte(file); // attributes
	readString(file, (char*)uniqueID, 3);
	for (int i=0; i<numRecords-1; i++)
	{
		offset = readLong(file);
		readByte(file); // attributes
		readString(file, (char*)uniqueID, 3);
		recSizes.add( (void*)(offset-lastOffset) );
		lastOffset = offset;
	}
	recSizes.add( (void*)( file.GetLength()-offset ) );

	readShort(file); // pad
	offset = 80+numRecords*8;


	PRecord record;
	PBottle* bottle;
	char* data;
	long len;
	for (i=0; i<numRecords; i++)
	{
		len = (long)recSizes.getAt(i);

		//int start = len;
		if (i == numRecords-1)
		{
			//end = (int)file.GetLength();
			//end = ((PFileRecord*)recList_.getAt(i+1))->localChunkID_;
		}
		else
		{
			//end = (long)recSizes.getAt(i+1);
		}

        data = new char[len+2];
        readString(file, data, len);
		//writeFile(r.name, end-start);
		//offset += len;


		//len = end-start;
		record.setBuffer(data, len);

		bottle = new PBottle;
		bottle->loadWithEvents(record);

		coll.add(bottle);
	}

	return true;
}

void PDatabaseFile::readRecord(CFile& file, PFileRecord* rec)
{
	rec->localChunkID_ = readLong(file);
	rec->attributes_ = readByte(file);
	readString(file, (char*)rec->uniqueID_, 3);
}

Boolean PDatabaseFile::writeByte(CFile& file, BYTE b)
{
	file.Write(&b, 1);
	return true;
}

Boolean PDatabaseFile::writeShort(CFile& file, short s)
{
	char* val = (char*)&s;

	char c = val[0];
	val[0] = val[1];
	val[1] = c;
    
	WORD aval = *(WORD*)val;

	file.Write(&aval, 2);

	return true;
}

Boolean PDatabaseFile::writeLong(CFile& file, long l)
{
	char* val = (char*)&l;
	DWORD aval = (DWORD)(val[0] << 24) + (DWORD)(val[1] << 16) + (DWORD)(val[2] << 8) + (DWORD)val[3];

	file.Write(&aval, 4);

	return true;
}

Boolean PDatabaseFile::writeString(CFile& file, char* buf)
{
	long len = strlen(buf);
	writeString(file, buf, len);
	return true;
}

Boolean PDatabaseFile::writeString(CFile& file, char* buf, long nb)
{
	file.Write(buf, nb);
	return true;
}

Boolean PDatabaseFile::write(PCollection& coll, const PString& name)
{
	CFile file;

	if (!file.Open(name.data(), CFile::modeWrite | CFile::modeCreate ))
	{
		return false;
	}


	long numRecords = coll.getSize();
	//long localChunkID;
	//BYTE attributes;



	char string[32];
	memset(string, 0, 32);
	strcpy(string, "cavistBottlesDB");
	//strcpy(string, "cavistDB");
	writeString(file, string, 32);
	
	attributes_ = (short)0x8000;
	writeShort(file, attributes_);

	version_ = 0;
	writeShort(file, version_);
	
	creationDate_ = 0x35223491;
	writeLong(file, creationDate_);
	
	modificationDate_ =	0x35226E89;
	writeLong(file, modificationDate_);
	
	lastBackupDate_ = 0;
	writeLong(file, lastBackupDate_);
	
	modificationNumber_ = 0x3C;
	writeLong(file, modificationNumber_);
	
	appInfoID_ = 0;
	writeLong(file, appInfoID_);
	
	sortInfoID_ = 0;
	writeLong(file, sortInfoID_);
	
	writeString(file, "BOTL", 4);

	writeString(file, "CVST", 4);

	uniqueIDSeed_ = 0;
	writeLong(file, uniqueIDSeed_);

	// RecordListType
	nextRecordListID_ = 0;
	writeLong(file, nextRecordListID_);

	numRecords = (short)coll.getSize();
	writeShort(file, (short)numRecords);

	long i;

	PBottle* bottle;
	PRecord rec;

	//char uniqueID[4]; uniqueID[0] = 0xD4; uniqueID[1] = 0xC0; uniqueID[2] = 0x05;
	char uniqueID[4]; uniqueID[0] = 0; uniqueID[1] = 0; uniqueID[2] = 0;

	long offset = 80+numRecords*8;
	rec.setBuffer(new char[1002], 1000);
	for (i=0; i<numRecords; i++)
	{
		//uniqueID[2]++;
		bottle = (PBottle*)coll.getAt(i);
		if (bottle == 0)
		{
			logLine("ERROR: The bottle is NULL");
			return false;
		}

		bottle->save(rec);
		writeRecordDef(file, offset, 0x40, uniqueID);
		offset = offset + rec.getSize();
	}
	writeShort(file, 0); // pad


	for (i=0; i<numRecords; i++)
	{
		bottle = (PBottle*)coll.getAt(i);
		if (bottle == 0)
		{
			logLine("The bottle is NULL");
			return false;
		}

		bottle->save(rec);

		writeRecordData(file, rec);
	}

	return true;
}

void PDatabaseFile::writeRecordDef(CFile& file, long localChunkID, BYTE attributes, char* uniqueID)
{
	writeLong(file, localChunkID);
	writeByte(file, attributes);
	writeString(file, (char*)uniqueID, 3);
}

void PDatabaseFile::writeRecordData(CFile& file, PRecord& rec)
{
	writeString(file, (char*)rec.getData(), rec.getSize());
}
